@@ -50,6 +50,9 @@ class Writer(object):
         self.output = StringIO()
         self.with_javascript = False
 
+    def is_at_global_level(self):
+        return self.level == 0
+
     def push(self):
         self.level += 1
 
@@ -178,6 +181,11 @@ class PythonToPythonJS(NodeVisitor):
         self._with_js = False
         self._typedefs = dict()  ## class name : typedef  (not pickled)
 
+        self._globals = dict()
+        self._global_typed_lists = dict()  ## global name : set  (if len(set)==1 then we know it is a typed list)
+        self._global_typed_dicts = dict()
+        self._global_typed_tuples = dict()
+
         self._custom_operators = {}
 
         self.setup_builtins()
@@ -250,21 +258,41 @@ class PythonToPythonJS(NodeVisitor):
             )
             pickle.dump( a, open(os.path.join(self._module_path, self._module+'.module'), 'wb') )
 
+    def _check_for_module(self, name):
+        if self._module_path and name+'.module' in os.listdir(self._module_path):
+            return True
+        else:
+            return False
+
+    def _load_module(self, name):
+        f = open( os.path.join(self._module_path, name+'.module'), 'rb' )
+        a = pickle.load( f ); f.close()
+        return a
+
+    def visit_Import(self, node):
+        for alias in node.names:
+            writer.write( '## import: %s :: %s' %(a.name,a.asname) )
+            raise SyntaxError('import with a namespace is not support yet, use "from module import *" instead')
+
     def visit_ImportFrom(self, node):
         if node.module in MINI_STDLIB:
             for n in node.names:
                 if n.name in MINI_STDLIB[ node.module ]:
                     writer.write( 'JS("%s")' %MINI_STDLIB[node.module][n.name] )
 
-        elif self._module_path and node.module+'.module' in os.listdir(self._module_path):
-            f = open( os.path.join(self._module_path, node.module+'.module'), 'rb' )
-            a = pickle.load( f ); f.close()
-            self._classes.update( a['classes'] )
-            self._class_attributes.update( a['class_attributes'] )
-            self._instance_attributes.update( a['instance_attributes'] )
-            self._decorator_class_props.update( a['decorator_class_props'] )
-            self._function_return_types.update( a['function_return_types'] )
-            self._class_parents.update( a['class_parents'] )
+        elif self._check_for_module( node.module ):
+            if node.names[0].name == '*':
+                a = self._load_module( node.module )
+                self._classes.update( a['classes'] )
+                self._class_attributes.update( a['class_attributes'] )
+                self._instance_attributes.update( a['instance_attributes'] )
+                self._decorator_class_props.update( a['decorator_class_props'] )
+                self._function_return_types.update( a['function_return_types'] )
+                self._class_parents.update( a['class_parents'] )
+            else:
+                raise SyntaxError('only "from module import *" is allowed')
+
+            writer.write('## import from: %s :: %s' %(node.module, [ (a.name,a.asname) for a in node.names]))
 
     def visit_Assert(self, node):
         ## hijacking "assert isinstance(a,A)" as a type system ##
@@ -296,10 +324,12 @@ class PythonToPythonJS(NodeVisitor):
             return 'get_attribute(dict, "__call__")([], JSObject(js_object=%s))' %b
 
     def visit_Tuple(self, node):
+        node.returns_type = 'tuple'
         a = '[%s]' % ', '.join(map(self.visit, node.elts))
         return 'get_attribute(tuple, "__call__")([], JSObject(js_object=%s))' %a
 
     def visit_List(self, node):
+        node.returns_type = 'list'
         a = '[%s]' % ', '.join(map(self.visit, node.elts))
         if self._with_js:
             return a
@@ -307,6 +337,7 @@ class PythonToPythonJS(NodeVisitor):
             return 'get_attribute(list, "__call__")([], JSObject(js_object=%s))' %a
 
     def visit_ListComp(self, node):
+        node.returns_type = 'list'
         writer.write('var(__comprehension__)')
         writer.write('__comprehension__ = JSArray()')
 
@@ -349,6 +380,10 @@ class PythonToPythonJS(NodeVisitor):
     def visit_In(self, node):
         return ' in '
 
+    def visit_NotIn(self, node):
+        #return ' not in '
+        raise RuntimeError('"not in" is only allowed in if-test: see method - visit_Compare')
+
     def visit_AugAssign(self, node):
         target = self.visit( node.target )
         op = '%s=' %self.visit( node.op )
@@ -426,8 +461,10 @@ class PythonToPythonJS(NodeVisitor):
                 self._class_attributes[ name ].add( item_name )  ## should this come before self.visit(item) ??
             elif isinstance(item, Pass):
                 pass
+            elif isinstance(item, ast.Expr) and isinstance(item.value, Str):  ## skip doc strings
+                pass
             else:
-                raise NotImplementedError
+                raise NotImplementedError( item )
 
         for prop_name in self._decorator_properties:
             getter = self._decorator_properties[prop_name]['get']
@@ -506,7 +543,7 @@ class PythonToPythonJS(NodeVisitor):
             writer.write('return %s' % self.visit(node.value))
 
         else:
-            raise RuntimeError
+            writer.write('return')  ## empty return
 
     def visit_BinOp(self, node):
         left = self.visit(node.left)
@@ -523,14 +560,14 @@ class PythonToPythonJS(NodeVisitor):
                 #return '%s( %s, %s )' %(op, left_operand, right_operand)
                 if op.decode('utf-8') in self._custom_operators:  ## swap name to python function
                     op = self._custom_operators[ op.decode('utf-8') ]
-                return '%s( [%s, %s] )' %(op, left_operand, right_operand)
+                return '%s( [%s, %s], JSObject() )' %(op, left_operand, right_operand)
 
         if isinstance(node.left, Name):
             typedef = self.get_typedef( node.left )
             if typedef and op in typedef.operators:
                 func = typedef.operators[ op ]
                 node.operator_overloading = func
-                return '''JS('%s( [%s, %s] )')''' %(func, left, right)  ## TODO double check this without wrapping in JS()
+                return '''JS('%s( [%s, %s], JSObject() )')''' %(func, left, right)  ## TODO double check this without wrapping in JS()
 
         return '%s %s %s' % (left, op, right)
 
@@ -583,18 +620,29 @@ class PythonToPythonJS(NodeVisitor):
         left = self.visit(node.left)
         comp = [ left ]
         for i in range( len(node.ops) ):
-            if isinstance(node.ops[i], ast.In):
+            if isinstance(node.ops[i], ast.In) or isinstance(node.ops[i], ast.NotIn):
                 if comp[-1] == left:
                     comp.pop()
                 else:
                     comp.append( ' and ' )
+
+                if isinstance(node.ops[i], ast.NotIn):
+                    comp.append( ' not (')
+
                 a = ( self.visit(node.comparators[i]), left )
-                if self._with_js:  ## this makes "if 'x' in Array" work like Python: "if 'x' in list" - TODO fix this for js-objects
-                    comp.append( '%s in %s or' %(a[1], a[0]) )  ## this is ugly, but it works
-                    comp.append( 'Object.hasOwnProperty(%s, "__contains__") and' %a[0])
+                if self._with_js:
+                    ## this makes "if 'x' in Array" work like Python: "if 'x' in list" - TODO fix this for js-objects
+                    ## note javascript rules are confusing: "1 in [1,2]" is true, this is because a "in test" in javascript tests for an index
+                    ## TODO double check this code
+                    comp.append( '%s in %s or' %(a[1], a[0]) )  ## this is ugly, will break with Arrays
+                    comp.append( 'Object.hasOwnProperty.call(%s, "__contains__") and' %a[0])
                     comp.append( "%s['__contains__'](%s)" %a )
                 else:
                     comp.append( "get_attribute(get_attribute(%s, '__contains__'), '__call__')([%s], JSObject())" %a )
+
+                if isinstance(node.ops[i], ast.NotIn):
+                    comp.append( ' )')  ## it is not required to enclose NotIn
+
             else:
                 comp.append( self.visit(node.ops[i]) )
                 comp.append( self.visit(node.comparators[i]) )
@@ -625,7 +673,7 @@ class PythonToPythonJS(NodeVisitor):
                 getter = typedef.properties[ node.attr ]['get']
                 if getter in self._function_return_types:
                     node.returns_type = self._function_return_types[getter]
-                return '%s( [%s] )' %(getter, node_value)
+                return '%s( [%s], JSObject() )' %(getter, node_value)
 
             elif node.attr in typedef.class_attributes and not typedef.check_for_parent_with( class_attribute=node.attr ) and node_value != 'self':
                 ## This optimization breaks when a subclass redefines a class attribute,
@@ -639,14 +687,14 @@ class PythonToPythonJS(NodeVisitor):
 
             elif '__getattr__' in typedef.methods:
                 func = typedef.get_pythonjs_function_name( '__getattr__' )
-                return '%s([%s, "%s"])' %(func, node_value, node.attr)
+                return '%s([%s, "%s"], JSObject())' %(func, node_value, node.attr)
 
             elif typedef.check_for_parent_with( property=node.attr ):
                 parent = typedef.check_for_parent_with( property=node.attr )
                 getter = parent.properties[ node.attr ]['get']
                 if getter in self._function_return_types:
                     node.returns_type = self._function_return_types[getter]
-                return '%s( [%s] )' %(getter, node_value)
+                return '%s( [%s], JSObject() )' %(getter, node_value)
 
             elif typedef.check_for_parent_with( class_attribute=node.attr ):
                 #return 'get_attribute(%s, "%s")' % (node_value, node.attr)  ## get_attribute is broken with grandparent class attributes
@@ -660,7 +708,7 @@ class PythonToPythonJS(NodeVisitor):
             elif typedef.check_for_parent_with( method='__getattr__' ):
                 parent = typedef.check_for_parent_with( method='__getattr__' )
                 func = parent.get_pythonjs_function_name( '__getattr__' )
-                return '%s([%s, "%s"])' %(func, node_value, node.attr)
+                return '%s([%s, "%s"], JSObject())' %(func, node_value, node.attr)
 
             else:
                 return 'get_attribute(%s, "%s")' % (node_value, node.attr)  ## TODO - double check this
@@ -683,7 +731,7 @@ class PythonToPythonJS(NodeVisitor):
         elif name in self._instances:  ## support x[y] operator overloading
             klass = self._instances[ name ]
             if '__getitem__' in self._classes[ klass ]:
-                return '__%s___getitem__( [%s, %s] )' % (klass, name, self.visit(node.slice))
+                return '__%s___getitem__([%s, %s], JSObject())' % (klass, name, self.visit(node.slice))
             else:
                 return 'get_attribute(%s, "__getitem__")([%s], JSObject())' % (
                     self.visit(node.value),
@@ -727,7 +775,7 @@ class PythonToPythonJS(NodeVisitor):
                 writer.write( '%s.%s=%s' %(target_value, target.attr, self.visit(node.value)) )
             elif typedef and target.attr in typedef.properties and 'set' in typedef.properties[ target.attr ]:
                 setter = typedef.properties[ target.attr ]['set']
-                writer.write( '%s( [%s, %s] )' %(setter, target_value, self.visit(node.value)) )
+                writer.write( '%s( [%s, %s], JSObject() )' %(setter, target_value, self.visit(node.value)) )
             elif typedef and target.attr in typedef.class_attributes:
                 writer.write( '''%s['__class__']['__dict__']['%s'] = %s''' %(target_value, target.attr, self.visit(node.value)))
             elif typedef and target.attr in typedef.attributes:
@@ -739,16 +787,16 @@ class PythonToPythonJS(NodeVisitor):
                 parent_setattr = typedef.check_for_parent_with( method='__setattr__' )
                 if parent_prop and 'set' in parent_prop.properties[target.attr]:
                     setter = parent_prop.properties[target.attr]['set']
-                    writer.write( '%s( [%s, %s] )' %(setter, target_value, self.visit(node.value)) )
+                    writer.write( '%s( [%s, %s], JSObject() )' %(setter, target_value, self.visit(node.value)) )
                 elif parent_classattr:
                     writer.write( "window['__%s_attrs']['%s'] = %s" %(parent_classattr.name, target.attr, self.visit(node.value)) )
                 elif parent_setattr:
                     func = parent_setattr.get_pythonjs_function_name( '__setattr__' )
-                    writer.write( '%s([%s, "%s", %s])' %(func, target_value, target.attr, self.visit(node.value)) )
+                    writer.write( '%s([%s, "%s", %s], JSObject() )' %(func, target_value, target.attr, self.visit(node.value)) )
 
                 elif '__setattr__' in typedef.methods:
                     func = typedef.get_pythonjs_function_name( '__setattr__' )
-                    writer.write( '%s([%s, "%s", %s])' %(func, target_value, target.attr, self.visit(node.value)) )
+                    writer.write( '%s([%s, "%s", %s], JSObject() )' %(func, target_value, target.attr, self.visit(node.value)) )
 
                 else:
                     code = 'set_attribute(%s, "%s", %s)' % (
@@ -761,7 +809,7 @@ class PythonToPythonJS(NodeVisitor):
             elif typedef and '__setattr__' in typedef.methods:
                 func = typedef.get_pythonjs_function_name( '__setattr__' )
                 log('__setattr__ in instance typedef.methods - func:%s target_value:%s target_attr:%s' %(func, target_value, target_attr))
-                writer.write( '%s([%s, "%s", %s])' %(func, target_value, target.attr, self.visit(node.value)) )
+                writer.write( '%s([%s, "%s", %s], JSObject() )' %(func, target_value, target.attr, self.visit(node.value)) )
 
 
             else:
@@ -800,7 +848,22 @@ class PythonToPythonJS(NodeVisitor):
             elif target.id in self._instances:
                 self._instances.pop( target.id )
 
-            writer.write('%s = %s' % (target.id, node_value))
+            if target.id in self._instances:
+                type = self._instances[ target.id ]
+                if writer.is_at_global_level():
+                    self._globals[ target.id ] = type
+                    if type == 'list':
+                        self._global_typed_lists[ target.id ] = set()
+                    elif type == 'tuple':
+                        self._global_typed_tuples[ target.id ] = set()
+                    elif type == 'dict':
+                        self._global_typed_dicts[ target.id ] = set()
+
+                    writer.write('%s = %s' % (target.id, node_value))
+                else:
+                    writer.write('%s = %s' % (target.id, node_value))
+            else:
+                writer.write('%s = %s' % (target.id, node_value))
 
         else:  # it's a Tuple
             id = self.identifier
@@ -846,7 +909,7 @@ class PythonToPythonJS(NodeVisitor):
                 a = ','.join(args)
                 return '%s( %s )' %( self.visit(node.func), a )
 
-        if isinstance(node.func, Name) and node.func.id in ('JS', 'toString', 'JSObject', 'JSArray', 'var', 'instanceof'):
+        if isinstance(node.func, Name) and node.func.id in ('JS', 'toString', 'JSObject', 'JSArray', 'var', 'instanceof', 'typeof'):
             args = list( map(self.visit, node.args) ) ## map in py3 returns an iterator not a list
             if node.func.id == 'var':
                 for k in node.keywords:
@@ -858,6 +921,20 @@ class PythonToPythonJS(NodeVisitor):
             args = ', '.join(args)
             return '%s(%s)' % (node.func.id, args)
         else:
+
+            ## check if pushing to a global typed list ##
+            if isinstance(node.func, ast.Attribute) and isinstance(node.func.value, Name) and node.func.value.id in self._globals:
+                gtype = self._globals[ node.func.value.id ]
+                if gtype == 'list' and node.func.attr == 'append':
+                    if isinstance(node.args[0], Name):
+                        if node.args[0].id in self._instances:
+                            gset = self._global_typed_lists[ node.func.value.id ]
+                            gset.add( self._instances[node.args[0].id])
+                            if len(gset) != 1:
+                                raise SyntaxError('global lists can only contain one type: instance "%s" is different' %node.args[0].id)
+                        else:
+                            raise SyntaxError('global lists can only contain one type: instance "%s" is unknown' %node.args[0].id)
+
             call_has_args_only = len(node.args) and not (len(node.keywords) or node.starargs or node.kwargs)
             call_has_args = len(node.args) or len(node.keywords) or node.starargs or node.kwargs
             name = self.visit(node.func)
@@ -875,17 +952,21 @@ class PythonToPythonJS(NodeVisitor):
                 self.identifier += 1
 
                 if name in ('list', 'tuple'):
-                    writer.append( '%s = JS("%s.__dict__.js_object")' % (args_name, args))
+                    #writer.append( '%s = JS("%s.__dict__.js_object")' % (args_name, args))
+                    if args:
+                        writer.append( '%s = %s.__dict__.js_object' % (args_name, args))  ## test this
+                    else:
+                        writer.append( '%s = []' %args_name )
                 else:
                     writer.append('%s = JSArray(%s)' % (args_name, args))
 
                 if node.starargs:
-                    writer.append('%s.push.apply(%s, %s)' % (args_name, args_name, self.visit(node.starargs)))
+                    writer.append('%s.push.apply(%s, %s.__dict__.js_object)' % (args_name, args_name, self.visit(node.starargs)))
                 writer.append('%s = JSObject(%s)' % (kwargs_name, kwargs))
 
                 if node.kwargs:
                     kwargs = self.visit(node.kwargs)
-                    code = "JS('for (var name in %s) { %s[name] = %s[name]; }')" % (kwargs, kwargs_name, kwargs)
+                    code = "JS('for (var name in %s) { %s[name] = %s.__dict__.js_object[name]; }')" % (kwargs, kwargs_name, kwargs)
                     writer.append(code)
 
             if call_has_args_only:
@@ -895,14 +976,25 @@ class PythonToPythonJS(NodeVisitor):
                 if name == 'dict':
                     return 'get_attribute(%s, "__call__")(%s, JSObject(js_object=%s))' % (name, args_name, kwargs_name)
                 elif name in ('list', 'tuple'):
-                    return 'get_attribute(%s, "__call__")([], JSObject(js_object=%s))' % (name, args_name)
+                    if len(node.args):
+                        return 'get_attribute(%s, "__call__")([], JSObject(js_object=%s))' % (name, args_name)
+                    else:
+                        return 'get_attribute(%s, "__call__")([], %s)' % (name, kwargs_name)
                 else:
                     return 'get_attribute(%s, "__call__")(%s, %s)' % (name, args_name, kwargs_name)
 
             elif name in self._classes or name in self._builtins:
                 return 'get_attribute(%s, "__call__")( JSArray(), JSObject() )' %name
 
-            else:  ## this could be a dangerous optimization ##
+            else:
+                ## this a slightly dangerous optimization,
+                ## because if the user is trying to create an instance of some class
+                ## and that class is define in an external binding,
+                ## and they forgot to put "from mylibrary import *" in their script (an easy mistake to make)
+                ## then this fails to call __call__ to initalize the instance,
+                ## and throws a confusing error:
+                ## Uncaught TypeError: Property 'SomeClass' of object [object Object] is not a function 
+                ## TODO - remove this optimization, or provide the user with a better error message.
                 return '%s( JSArray(), JSObject() )' %name
 
     def visit_Lambda(self, node):
@@ -986,11 +1078,31 @@ class PythonToPythonJS(NodeVisitor):
                 a = ','.join( local_vars-global_vars )
                 writer.write('var(%s)' %a)
 
+
+
         if not self._with_js and (len(node.args.defaults) or len(node.args.args) or node.args.vararg or node.args.kwarg):
+            # First check the arguments are well formed 
+            # ie. that this function is not a callback of javascript code
+            writer.write("""if (JS('args instanceof Array') and JS("{}.toString.call(kwargs) === '[object Object]'") and arguments.length == 2):""")
+            # XXX: there is bug in the underlying translator preventing me to write the condition
+            # in a more readble way... something to do with brakects...
+            writer.push()
+            writer.write('pass')  # do nothing if it's not called from javascript
+            writer.pull()
+            writer.write('else:')
+            writer.push()
+            # If it's the case, move use ``arguments`` to ``args`` 
+            writer.write('args = Array.prototype.slice.call(arguments)')
+            # This means you can't pass keyword argument from javascript but we already knew that
+            writer.write('kwargs = JSObject()')
+            writer.pull()
+            # done with pythonjs function used as callback of Python code 
+
             # new pythonjs' python function arguments handling
             # create the structure representing the functions arguments
             # first create the defaultkwargs JSObject
             writer.write('var(signature, arguments)')
+
             L = len(node.args.defaults)
             kwargsdefault = map(lambda x: keyword(self.visit(x[0]), x[1]), zip(node.args.args[-L:], node.args.defaults))
             kwargsdefault = Call(
@@ -1045,23 +1157,38 @@ class PythonToPythonJS(NodeVisitor):
             self._function_return_types[ node.name ] = self._return_type
 
         writer.pull()
+        ## note, in javascript function.name is a non-standard readonly attribute,
+        ## the compiler creates anonymous functions with name set to an empty string.
+        writer.write('%s.NAME = "%s"' %(node.name,node.name))
+
+        writer.write( '%s.args_signature = [%s]' %(node.name, ','.join(['"%s"'%n.id for n in node.args.args])) )
+        defaults = ['%s:%s'%(self.visit(x[0]), self.visit(x[1])) for x in zip(node.args.args[-len(node.args.defaults):], node.args.defaults) ]
+        writer.write( '%s.kwargs_signature = {%s}' %(node.name, ','.join(defaults)) )
+
         if self._with_js and with_js_decorators:
-            ## these with-js functions are assigned to a some objects prototype,
-            ## here we assume that they depend on the special "this" variable,
-            ## therefore this function can not be marked as f.pythonscript_function,
-            ## because we need get_attribute(f,'__call__') to dynamically bind "this"
             for dec in with_js_decorators:
-                assert '.prototype.' in dec
-                writer.write( '%s=%s'%(dec,node.name) )
-        elif self._with_js:  ## this is just an optimization so we can avoid making wrappers at runtime
-            writer.write('%s.pythonscript_function=true'%node.name)
-        else:
+                if '.prototype.' in dec:
+                    ## these with-js functions are assigned to a some objects prototype,
+                    ## here we assume that they depend on the special "this" variable,
+                    ## therefore this function can not be marked as f.pythonscript_function,
+                    ## because we need get_attribute(f,'__call__') to dynamically bind "this"
+                    writer.write( '%s=%s'%(dec,node.name) )
+                else:  ## TODO fix with-javascript decorators
+                    writer.write( '%s = get_attribute(%s,"__call__")( [%s], {} )' %(node.name, dec, node.name))
+
+        ## Gotcha, this broke calling a "with javascript:" defined function from pythonjs, 
+        ## because get_attribute thought it was dealing with a pythonjs function and was
+        ## calling the function in the normal pythonjs way, ie. func( [args], {} )
+        #elif self._with_js:  ## this is just an optimization so we can avoid making wrappers at runtime
+        #    writer.write('%s.pythonscript_function=true'%node.name)
+
+        if not self._with_js:
             writer.write('%s.pythonscript_function=True'%node.name)
 
         # apply decorators
         for decorator in decorators:
             assert not self._with_js
-            writer.write('%s = %s(create_array(%s))' % (node.name, self.visit(decorator), node.name))
+            writer.write('%s = get_attribute(%s,"__call__")( [%s], {} )' % (node.name, self.visit(decorator), node.name))
 
     def visit_Continue(self, node):
         if self._with_js:
@@ -1078,7 +1205,12 @@ class PythonToPythonJS(NodeVisitor):
             map(self.visit, node.body)
             writer.pull()
         else:
-            self._for_iterator_target = node.target.id
+
+            ## TODO else remove node.target.id from self._instances
+            if isinstance(node.iter, Name) and node.iter.id in self._global_typed_lists:
+                self._instances[ node.target.id ] = list( self._global_typed_lists[ node.iter.id ] )[0]
+
+            self._for_iterator_target = node.target.id  ## this could break with nested for loops
             writer.write('var(__iterator__, %s)' % node.target.id)
             writer.write('__iterator__ = get_attribute(get_attribute(%s, "__iter__"), "__call__")(JSArray(), JSObject())' % self.visit(node.iter))
             writer.write('try:')
@@ -1109,7 +1241,26 @@ class PythonToPythonJS(NodeVisitor):
             map(self.visit, node.body)
             writer.with_javascript = False
             self._with_js = False
+        elif isinstance( node.context_expr, Name ) and node.context_expr.id == 'python':
+            if not self._with_js:
+                raise SyntaxError('"with python:" is only used inside of a "with javascript:" block')
+            self._with_js = False
+            writer.with_javascript = False
+            map(self.visit, node.body)
+            writer.with_javascript = True
+            self._with_js = True
 
+        elif isinstance( node.context_expr, Name ) and node.optional_vars and isinstance(node.optional_vars, Name) and node.optional_vars.id == 'jsobject':
+            instance_name = node.context_expr.id
+            for n in node.body:
+                if isinstance(n, ast.Expr) and isinstance(n.value, Name):
+                    attr_name = n.value.id
+                    writer.write('%s.%s = get_attribute(%s, "%s")'%(instance_name, attr_name, instance_name, attr_name))
+                else:
+                    raise SyntaxError('invalid statement inside of "with x as jsobject:" block')
+
+        else:
+            raise SyntaxError('improper use of "with" statement')
 
 def main(script):
     input = parse(script)
